import 'package:flutter/material.dart';

import '../../consts.dart';

class InfoLayer extends StatefulWidget {

  const InfoLayer({
    Key? key,
  }) : super(key: key);

  @override
  InfoLayerState createState() => InfoLayerState();
}

class InfoLayerState extends State<InfoLayer> {
  @override
  void initState() {
    super.initState();
  }

  // _buildLegendItem renders a colored circle and text to form a legend
  Widget _legendItem(Color color, String text) {
    return Row(
      children: [
        Container(
          width: 20,
          height: 20,
          decoration: BoxDecoration(
            color: color,
            shape: BoxShape.circle,
          ),
        ),
        const SizedBox(width: 8),
        Text(
          text,
          style: const TextStyle(
            fontSize: 14,
            color: Colors.white,
          ),
        ),
      ],
    );
  }

  /// Builds an additional legend to explain the colors
  Widget _buildLegend() {
    return Column(
      children: [
        _legendItem(const Color(0xffff0000), "Very unsafe"),
        const SizedBox(height: 10),
        _legendItem(const Color(0xffff6a00), "Unsafe"),
        const SizedBox(height: 10),
        _legendItem(const Color(0xFFb1ff00), "Safe"),
        const SizedBox(height: 10),
        _legendItem(const Color(0xFF00d6ff), "Very safe"),
        const SizedBox(height: 10),
      ],
    );
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      padding: const EdgeInsets.all(40),
      color: Colors.black.withOpacity(0.8),
      child: Column(
        mainAxisAlignment: MainAxisAlignment.center, // Align contents vertically centered
        children: [
          Text(
            'Color categorization',
            style: subHeadingStyle,
          ),
          const SizedBox(height: 20),
          Text(
            'The map shows the safety of applying x kg per y m^2',
            style: regTextStyle,
            textAlign: TextAlign.center,
          ),
          const SizedBox(height: 30),
          _buildLegend(),
          const SizedBox(height: 30),
          Text(
            'Placeholder for other information...',
            style: smallTextStyle,
            textAlign: TextAlign.center,
          ),
        ],
      ),
    );
  }
}
